//
// 2nd order delta-sigma modulator
// (2-level quantization / 8-Tap Moving Average)
//
//   designed by K.Yoshizawa 2008.08.25
//
#include <p30f2012.h>

	unsigned char freqNum[5];
	int data[100];
	int i;
	long temp;

	unsigned char freqSel;
	unsigned char priCount;
	unsigned char pointer;
	int aReg;
	int bReg;
	unsigned char portBBuf;

int main(void) {
// i-cycle freqency  : 117.92MHz /   4 = 29.48MHz
// sampling freqency : 29.48MHz  /  74 = 398.4KHz
// timer1 set
	PR1   = 0x0049;	//  74d = 4AH
	T1CON = 0x8000;

// I/O port set
	TRISF = 0xFFDF;	// PF5 ... output
	PORTF = 0xFFFF;	// PF5 initial Hi
	TRISB = 0xFF00;	// PB7 ~ PB0 ... output
	PORTB = 0x0055;	// PB7 ~ PB0 initial 55H

// sin data table (signed 16-bit data / 100 point)
	data[0]  = 0;
	data[1]  = 2057;
	data[2]  = 4107;
	data[3]  = 6140;
	data[4]  = 8149;
	data[5]  = 10126;
	data[6]  = 12062;
	data[7]  = 13952;
	data[8]  = 15786;
	data[9]  = 17557;
	data[10] = 19260;
	data[11] = 20886;
	data[12] = 22431;
	data[13] = 23886;
	data[14] = 25247;
	data[15] = 26509;
	data[16] = 27666;
	data[17] = 28714;
	data[18] = 29648;
	data[19] = 30466;
	data[20] = 31163;
	data[21] = 31738;
	data[22] = 32187;
	data[23] = 32509;
	data[24] = 32702;
	data[25] = 32767;
	data[26] = 32702;
	data[27] = 32509;
	data[28] = 32187;
	data[29] = 31738;
	data[30] = 31163;
	data[31] = 30466;
	data[32] = 29648;
	data[33] = 28714;
	data[34] = 27666;
	data[35] = 26509;
	data[36] = 25247;
	data[37] = 23886;
	data[38] = 22431;
	data[39] = 20886;
	data[40] = 19260;
	data[41] = 17557;
	data[42] = 15786;
	data[43] = 13952;
	data[44] = 12062;
	data[45] = 10126;
	data[46] = 8149;
	data[47] = 6140;
	data[48] = 4107;
	data[49] = 2057;
	data[50] = 0;
	data[51] = -2057;
	data[52] = -4107;
	data[53] = -6140;
	data[54] = -8149;
	data[55] = -10126;
	data[56] = -12062;
	data[57] = -13952;
	data[58] = -15786;
	data[59] = -17557;
	data[60] = -19260;
	data[61] = -20886;
	data[62] = -22431;
	data[63] = -23886;
	data[64] = -25247;
	data[65] = -26509;
	data[66] = -27666;
	data[67] = -28714;
	data[68] = -29648;
	data[69] = -30466;
	data[70] = -31163;
	data[71] = -31738;
	data[72] = -32187;
	data[73] = -32509;
	data[74] = -32702;
	data[75] = -32767;
	data[76] = -32702;
	data[77] = -32509;
	data[78] = -32187;
	data[79] = -31738;
	data[80] = -31163;
	data[81] = -30466;
	data[82] = -29648;
	data[83] = -28714;
	data[84] = -27666;
	data[85] = -26509;
	data[86] = -25247;
	data[87] = -23886;
	data[88] = -22431;
	data[89] = -20886;
	data[90] = -19260;
	data[91] = -17557;
	data[92] = -15786;
	data[93] = -13952;
	data[94] = -12062;
	data[95] = -10126;
	data[96] = -8149;
	data[97] = -6140;
	data[98] = -4107;
	data[99] = -2057;

// sin data gain control
	for(i = 0;i<100;i++) {
		temp = (long)data[i];
		temp = (temp * 7) >> 3;		// 7/8
		data[i] = (int)(temp >> 4);	// truncate to 12-bit
	}

// sin data read rate table set
	freqNum[0] =  2;	// 400KHz / 100 /  2 =  2KHz
	freqNum[1] =  4;	// 400KHz / 100 /  4 =  1KHz
	freqNum[2] =  8;	// 400KHz / 100 /  8 = 500Hz
	freqNum[3] = 20;	// 400KHz / 100 / 20 = 200Hz

// initialize variable
	freqSel  = 3;
	priCount = 0;
	pointer  = 0;
	aReg     = 0;
	bReg     = 0;
	portBBuf = 0x0055;

// main loop
	while(1) {							// infinite loop
		while(IFS0bits.T1IF == 0) {}	// timer1 flag ?
		IFS0bits.T1IF = 0;				// clear timer1 flag
// *** use ext trigger (400KHz) ... option ***
//		while(IFS0bits.INT0IF == 0) {}	// int0 flag ?
//		IFS0bits.INT0IF = 0;			// clear int0 flag

		PORTFbits.RF5 = 1;				// monitor LED off
		PORTB = portBBuf;				// delta-sigma output from buffer

		freqSel = (PORTF >> 2) & 0x03;	// read frequency select sw
		priCount = priCount + 1;		// frequency pri-scaller
		if (priCount >= freqNum[freqSel] ) {
			priCount = 0;

			pointer = pointer + 1;		// pointer of sin table
			if (pointer >= 100) {
				pointer = 0; }
		}

// delta-sigma process
		aReg = aReg + data[pointer];	// integrate
		bReg = bReg + aReg;
		if (bReg > 0) {					// quantization
			aReg = aReg - 2048;			// differential
			bReg = bReg - 2048;
			portBBuf = (portBBuf << 1) | 1;	// shift-reg. for moving average
		} else {
			aReg = aReg + 2048;			// differential
			bReg = bReg + 2048;
			portBBuf = portBBuf << 1;	// shift-reg. for moving average
		}

		PORTFbits.RF5 = 0;				// monitor LED on
	}
}
